/* Emperor - an X windows strategy game for 1-6 players
   Copyright (C) 1998 Frank Rustemeyer.
   This file is part of the Emperor package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   If you have changes and you want them to be included in a future
   release of Emperor, please e-mail them to mambres@mud.tap.de.

   This file implements the classes needed for network communication. 

   04/11/27, Frank Rustemeyer <mambres@mud.tap.de>: Released.
   04/11/28, Frank Rustemeyer <mambres@mud.tap.de>: Bugfix: active sockets must be nonblocking
*/

#include "empx.h"

#include <sys/types.h> // socket()
#include <sys/socket.h> // socket(), inet_ntoa(), inet_addr()
#include <netinet/in.h> // htons(), htonl()
#include <string.h> // memset(), memcpy()
#include <fcntl.h> // fcntl()
#include <unistd.h> // close(), read()
#include <netdb.h> // gethostbyname()
#include <netinet/in.h> // inet_ntoa(), inet_addr()
#include <arpa/inet.h> // inet_ntoa(), inet_addr()

// These are for the socket() call:
#define DOMAIN AF_INET
#define SOCKTYPE SOCK_STREAM
#define PROTOCOL 0

// These are for listen():
#define NUM_CONNECTIONS 2

//*************************************active_socket********

active_socket::active_socket() : basic_socket()
{   counter = 0;
    }
 
active_socket::active_socket(const char *hname, int port)
{   struct in_addr addr;
    struct sockaddr_in sin;
    struct hostent *host;

    counter = 0;
    sock = -1;

    if ((*hname < '0') || (*hname > '9'))
    {   host = gethostbyname(hname);
        if (host == NULL) return;
        hname = (char *) inet_ntoa(*(struct in_addr *) host->h_addr_list[0]);
        }

    addr.s_addr = inet_addr(hname);
    if ((int)addr.s_addr == -1) return; // couldn't parse address

    memset((char *)&sin, 0, sizeof(sin));
    sin.sin_family = DOMAIN;
    sin.sin_port = htons(port);
    memcpy((char *)&sin.sin_addr, &addr, sizeof(addr));

    sock = socket(DOMAIN, SOCKTYPE, PROTOCOL);
    if (sock < 0) return;

    if (connect(sock, (struct sockaddr *)&sin, sizeof(sin)) < 0)
    {   sock = -1;
        return;
        }

//    fcntl(sock, F_SETFL, O_NONBLOCK);
    }

char * active_socket::read_socket(void)
{   int r;
  
    if (!active()) return (char *) NULL;

    do
    {   r = read(sock, stringbuf + counter, 1);
        if (r < 1) return (char *) NULL;
        if (*(stringbuf + counter) == '\n')
             *(stringbuf + counter) = 0;
        counter++;
        if (counter == BUFSIZE - 1)
        {   *(stringbuf+counter) = 0;
            counter++;
            }
        }
    while (*(stringbuf + counter - 1));
    counter = 0;
    return stringbuf;
    }

void active_socket::write_socket(char *s)
{   if (write(sock, s, strlen(s)) == -1)
        sock = -1;
    }

//*************************************basic_socket********

basic_socket::basic_socket()
{   sock = -1; 
    }

basic_socket::~basic_socket()
{   if (sock > -1) close(sock);
    }

void basic_socket::close_me(void)
{
    close(sock);
    sock = -1;
    }

//******************************listen_socket*************

listen_socket::listen_socket(int port)
{   sock = socket(AF_INET, SOCK_STREAM, 0);
    if (sock < 0) return;
    fcntl(sock, F_SETFL, O_NONBLOCK);
    bind_me(port);
    if (sock < 0) return;
    start_listen();
    }

int listen_socket::bind_me (int port)
{   struct sockaddr_in s;

    if (sock < 0) return 0;

    memset( (char *) &s, 0, sizeof(s) );
    s.sin_port = htons(port);
    s.sin_addr.s_addr = htonl(INADDR_ANY);
    s.sin_family = AF_INET;

    if (bind(sock, (struct sockaddr *) &s, sizeof(s)) < 0)
    {   close(sock);
        sock = -1; /* mark error */
        return 0;
        }
 
    return 1;
    }

int listen_socket::start_listen(void)
{   if (sock < 0) return 0;

    if (listen(sock, NUM_CONNECTIONS))
    {   close(sock);
        sock = -1; /* mark error */
        return 0;
        }

    return 1;
    }
 
class active_socket * listen_socket::accept_connection(void)
{   struct sockaddr_in from;
    int fd;
    socklen_t size;
    class active_socket *newsock;

    size = sizeof(from);
    fd = accept(sock, (struct sockaddr *) &from, &size);

    if (fd == -1) return (active_socket *) NULL;

    fcntl(fd, F_SETFL, O_NONBLOCK);
    newsock = new active_socket();
    newsock->set_fd(fd);
    return newsock;
    } 

